/* wrappi
 * Copyright (C) 2011-2012 Red Hat Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#ifndef WRAPPI_INTERNAL_H_
#define WRAPPI_INTERNAL_H_

#include <string.h>
#include <errno.h>

#define STREQ(a,b) (strcmp((a),(b)) == 0)
#define STRCASEEQ(a,b) (strcasecmp((a),(b)) == 0)
#define STRNEQ(a,b) (strcmp((a),(b)) != 0)
#define STRCASENEQ(a,b) (strcasecmp((a),(b)) != 0)
#define STREQLEN(a,b,n) (strncmp((a),(b),(n)) == 0)
#define STRCASEEQLEN(a,b,n) (strncasecmp((a),(b),(n)) == 0)
#define STRNEQLEN(a,b,n) (strncmp((a),(b),(n)) != 0)
#define STRCASENEQLEN(a,b,n) (strncasecmp((a),(b),(n)) != 0)
#define STRPREFIX(a,b) (strncmp((a),(b),strlen((b))) == 0)

struct wrap_internal_h {
  char *error;          /* Last error on this handle, NULL is none. */
  int errnum;           /* errno, or 0 if there was no errno set. */
  const char *error_func; /* Function where the error occurred, if known. */
};

struct wrap_h {
  /* The "internal" part of the handle must always appear at the
   * beginning of the handle structure.  For the remote case, this
   * contains a cut-down handle which is what non-local entry points
   * are permitted to touch.  For the local case, the full handle is
   * cast to (struct wrap_internal_h *).
   */
  struct wrap_internal_h internal;

  /* Fields that follow the 'internal' structure only exist
   * on the local side of a connection.
   */

  /* Connection URL.  If scheme = NULL, it means we're using the local
   * code.
   */
  const char *scheme;
  const char *hostname;
};

/* Declare an error, setting the error field in the handle. */
#define set_error(fs...) \
  wrap_int_set_error ((struct wrap_internal_h *)w, __func__, fs)
#define set_error_errno(fs...) \
  wrap_int_set_error_errno ((struct wrap_internal_h *)w, (errno), __func__, fs)

extern void wrap_int_set_error (struct wrap_internal_h *w, const char *func, const char *fs, ...);
extern void wrap_int_set_error_errno (struct wrap_internal_h *w, int errnum, const char *func, const char *fs, ...);

#endif /* WRAPPI_INTERNAL_H_ */
